@@ -1,13 +1,13 @@
 import ssl
 import sys
 from types import TracebackType
-from typing import Iterable, Iterator, List, Optional, Type
+from typing import Iterable, Iterator, Iterable, List, Optional, Type
 
 from .._exceptions import ConnectionNotAvailable, UnsupportedProtocol
 from .._models import Origin, Request, Response
 from .._synchronization import Event, Lock
 from ..backends.sync import SyncBackend
-from ..backends.base import NetworkBackend
+from ..backends.base import SOCKET_OPTION, NetworkBackend
 from .connection import HTTPConnection
 from .interfaces import ConnectionInterface, RequestInterface
 
@@ -31,7 +31,8 @@ class RequestStatus:
     def wait_for_connection(
         self, timeout: Optional[float] = None
     ) -> ConnectionInterface:
-        self._connection_acquired.wait(timeout=timeout)
+        if self.connection is None:
+            self._connection_acquired.wait(timeout=timeout)
         assert self.connection is not None
         return self.connection
 
@@ -53,6 +54,7 @@ class ConnectionPool(RequestInterface):
         local_address: Optional[str] = None,
         uds: Optional[str] = None,
         network_backend: Optional[NetworkBackend] = None,
+        socket_options: Optional[Iterable[SOCKET_OPTION]] = None,
     ) -> None:
         """
         A connection pool for making HTTP requests.
@@ -80,6 +82,8 @@ class ConnectionPool(RequestInterface):
                 `local_address="::"` will connect using an `AF_INET6` address (IPv6).
             uds: Path to a Unix Domain Socket to use instead of TCP sockets.
             network_backend: A backend instance to use for handling network I/O.
+            socket_options: Socket options that have to be included
+             in the TCP socket when the connection was established.
         """
         self._ssl_context = ssl_context
 
@@ -108,6 +112,7 @@ class ConnectionPool(RequestInterface):
         self._network_backend = (
             SyncBackend() if network_backend is None else network_backend
         )
+        self._socket_options = socket_options
 
     def create_connection(self, origin: Origin) -> ConnectionInterface:
         return HTTPConnection(
@@ -120,6 +125,7 @@ class ConnectionPool(RequestInterface):
             local_address=self._local_address,
             uds=self._uds,
             network_backend=self._network_backend,
+            socket_options=self._socket_options,
         )
 
     @property
@@ -208,15 +214,15 @@ class ConnectionPool(RequestInterface):
             raise UnsupportedProtocol(
                 "Request URL is missing an 'http://' or 'https://' protocol."
             )
-        if scheme not in ("http", "https"):
+        if scheme not in ("http", "https", "ws", "wss"):
             raise UnsupportedProtocol(
                 f"Request URL has an unsupported protocol '{scheme}://'."
             )
 
         status = RequestStatus(request)
-        self._requests.append(status)
 
         with self._pool_lock:
+            self._requests.append(status)
             self._close_expired_connections()
             self._attempt_to_acquire_connection(status)
 
@@ -229,8 +235,11 @@ class ConnectionPool(RequestInterface):
                 # If we timeout here, or if the task is cancelled, then make
                 # sure to remove the request from the queue before bubbling
                 # up the exception.
-                self._requests.remove(status)
-                raise exc
+                with self._pool_lock:
+                    # Ensure only remove when task exists.
+                    if status in self._requests:
+                        self._requests.remove(status)
+                    raise exc
 
             try:
                 response = connection.handle_request(request)
@@ -273,11 +282,10 @@ class ConnectionPool(RequestInterface):
         assert status.connection is not None
         connection = status.connection
 
-        if status in self._requests:
-            self._requests.remove(status)
-
         with self._pool_lock:
             # Update the state of the connection pool.
+            if status in self._requests:
+                self._requests.remove(status)
 
             if connection.is_closed() and connection in self._pool:
                 self._pool.remove(connection)
@@ -301,19 +309,11 @@ class ConnectionPool(RequestInterface):
         Close any connections in the pool.
         """
         with self._pool_lock:
-            requests_still_in_flight = len(self._requests)
-
             for connection in self._pool:
                 connection.close()
             self._pool = []
             self._requests = []
 
-            if requests_still_in_flight:
-                raise RuntimeError(
-                    f"The connection pool was closed while {requests_still_in_flight} "
-                    f"HTTP requests/responses were still in-flight."
-                )
-
     def __enter__(self) -> "ConnectionPool":
         return self
 
@@ -349,6 +349,6 @@ class ConnectionPoolByteStream:
     def close(self) -> None:
         try:
             if hasattr(self._stream, "close"):
-                self._stream.close()  # type: ignore
+                self._stream.close()
         finally:
             self._pool.response_closed(self._status)
